#functions for calculating rate of succession using state-space model (random walk with Poisson observation)
#this is for discrete time with equally-spaced observations

#input a matrix: rows are years, cols are species
#and number of reps
#and maxit, max number of optimization iterations
#For each rep, sample from conditional distribution of true log abundance given all the data for each species
#Calculate growth rate from simulated data
#Use growth rates to get rbar and sr
#return list containing
#object rstats: list containing
#srmean: mean rate of succession
#sr025 and sr975: 0.025 and 0.975 quantiles of rate of succession
#rbarmean: mean growth rate
#rbar025 and rbar975: 0.025 and 0.975 quantiles of growth rate
#rnormmean: mean norm of growth rate vector
#rnorm025 and rnorm975: 0.025 and 0.975 quantiles of norm of growth rate vector
#cossqthetamean: mean cosine^2 of angle between vector u and vector r (proportion of change that is along line of equal growth rates)
#cossqtheta025 and cossqtheta975: 0.025 and 0.975 quantiles of cosine^2 of angle
#cossqthetadiffmean: mean cosine^2 of angle between vector of change in u and vectorof change in r (proportion of change in rates of change that is along a contour of sr)
#cossqthetadiff025 and cossqthetadiff975: 0.025 and 0.975 quantiles of cosine^2 of angle between vector of change in u and vector of change in r
#diffunormmean and diffrnormmean are means of norms of projection of change in growth rate vector onto line of equal growth rates, and change in growth rate vector
#
#and xmean, an array (years x species) of fitted log abundances
#and convs, vector of optimization convergence results (0 for success)
getsucc <- function(data,reps,maxit){
	nspp <- dim(data)[2]
	nyears <- dim(data)[1]
	simrdata <- array(dim=c(nyears-1,reps,nspp))
	xmean <- array(dim=c(nyears,nspp))
	convs <- array(dim=c(nspp,1))
	for(i in 1:nspp){
      	    ssfit <- simss(data[,i],reps,maxit)
	    simrdata[,,i] <- ssfit$rsim
	    xmean[,i] <- ssfit$yfit
	    convs[i] <- ssfit$conv
	}
	rstats <- getrstats(simrdata)
	return(list(rstats=rstats,xmean=xmean,convs=convs))
}

#negative log likelihood
fSssm <- function(xx,sssm)
#parameter W_1
{
	Wmat(sssm) <- function(tt,x,phi){return(exp(xx))}
	sssm.res <- kfs(sssm)
	return(-sssm.res$loglik)
}

#optimize parameters of state-space model
#input an unoptimized model Sssm and maxit (max iterations)
#return the optimized model (not filtered or smoothed) and conv (convergence value) and V (variance of parameter estimates)
getssm <- function(Sssm,maxit)
{
	optpars <- optim(0,fn=fSssm,sssm=Sssm,method="Brent",control=list(trace=1,maxit=maxit),hessian=TRUE,lower=-10,upper=10)
	Wmat(Sssm) <- function(tt,x,phi){return(exp(optpars$par))}
	V <- 1/optpars$hessian #asymptotic estimate of variance of Wmat
	return(list(conv=optpars$convergence,Sssm=Sssm,V=V))
}

#fit state-space model and simulate from conditional distribution of true abundance in state-space model with random walk and Poisson observation
#observations y (must be equally spaced)
#reps is number of reps to simulate
#maxit is max optimization iterations to use
#return list containing
#rsim: array of growth rates (rows are years, cols are reps)
#yfit: fitted log abundance
#conv: did optimization converge? (0 for success)
#this version initializes with first observation
simss <- function(y,reps,maxit)
{
	nyears <- length(y)
	Sssm <- ssm(y ~ tvar(1),family=poisson(link="log"),C0=1e6,m0=y[1],fit=TRUE)
	Sssmfit <- getssm(Sssm,maxit)
	#sample from conditional distribution of state given data, with parameter uncertainty
	sdata <- array(dim=c(nyears,reps))
	for(r in 1:reps){
	      Wr <- exp(rnorm(1,mean=0,sd=sqrt(Sssmfit$V))) #single sample from sampling distribution of Wmat
      	      Wmat(Sssmfit$Sssm) <- function(tt,x,phi){return(Wr)}
	      sfilt <- kfilter(Sssmfit$Sssm)
      	      sdata[,r] <- ksimulate(sfilt,N=1) #single sample from conditional distribution of true state given sampled value of Wmat
	}

        #########
        #print("TEST CODE: MAKING ALL REPS IDENTICAL!")
        #for(r in 2:reps){
        #  sdata[,r] <- sdata[,1] #make all reps identical
        #}
        #########

	rsim <- apply(sdata,2,diff)
	rsim <- array(rsim,dim=c(nyears-1,reps))
	Sssm.res <- kfs(Sssmfit$Sssm) #smooth to get fit
	yfit <- Sssm.res$m
	return(list(rsim=rsim,yfit=yfit,conv=Sssmfit$conv))
}

#get statistics based on growth rates
#simrdata is an array (years x reps x species) of growth rates
#return list containing
#srmean: mean rate of succession
#sr025 and sr975: 0.025 and 0.975 quantiles of rate of succession
#rbarmean: mean growth rate
#rbar025 and rbar975: 0.025 and 0.975 quantiles of growth rate
#rnormmean: mean norm of growth rate vector
#rnorm025 and rnorm975: 0.025 and 0.975 quantiles of norm of growth rate vector
#cossqthetamean: mean cosine^2 of angle between vector u and vector r (proportion of change that is along line of equal growth rates)
#cossqtheta025 and cossqtheta975: 0.025 and 0.975 quantiles of cosine^2 of angle
#cossqthetadiffmean: mean cosine^2 of angle between vector of change in u and vectorof change in r (proportion of change in rates of change that is along a contour of sr)
#cossqthetadiff025 and cossqthetadiff975: 0.025 and 0.975 quantiles of cosine^2 of angle between vector of change in u and vector of change in r
#diffunormmean and diffrnormmean are means of norms of projection of change in growth rate vector onto line of equal growth rates, and change in growth rate vector
getrstats <- function(simrdata)
{
  nspp <- dim(simrdata)[3]
  rbar <- apply(simrdata,c(1,2),mean)
  
  rbarmean <- apply(rbar,1,mean)
  rbar025 <- apply(rbar,1,quantile,0.025)
  rbar975 <- apply(rbar,1,quantile,0.975)

  sr <- apply(simrdata,c(1,2),sd)
  srmean <- apply(sr,1,mean)
  sr025 <- apply(sr,1,quantile,0.025)
  sr975 <- apply(sr,1,quantile,0.975)

  rnorm <- apply(simrdata,c(1,2),function(x) sqrt(sum(x^2)))
  rnormmean <- apply(rnorm,1,mean)
  rnorm025 <- apply(rnorm,1,quantile,0.025)
  rnorm975 <- apply(rnorm,1,quantile,0.975)

  cossqtheta <- nspp*(rbar/rnorm)^2
  cossqthetamean <- apply(cossqtheta,1,mean)
  cossqtheta025 <- apply(cossqtheta,1,quantile,0.025)
  cossqtheta975 <- apply(cossqtheta,1,quantile,0.975)

  diffrbar <- apply(rbar,2,diff)
  diffunorm <- sqrt(nspp)*abs(diffrbar)
  diffr <- apply(simrdata,c(2,3),diff)
  diffrnorm <- apply(diffr,c(1,2),function(x) sqrt(sum(x^2)))
  cossqthetadiff <- (diffunorm/diffrnorm)^2
  cossqthetadiffmean <- apply(cossqthetadiff,1,mean)
  cossqthetadiff025 <- apply(cossqthetadiff,1,quantile,0.025)
  cossqthetadiff975 <- apply(cossqthetadiff,1,quantile,0.975)

  diffunormmean <- apply(diffunorm,1,mean)
  diffrnormmean <- apply(diffrnorm,1,mean)
  
  return(list(srmean=srmean,sr025=sr025,sr975=sr975,rbarmean=rbarmean,rbar025=rbar025,rbar975=rbar975,rnormmean=rnormmean,rnorm025=rnorm025,rnorm975=rnorm975,cossqthetamean=cossqthetamean,cossqtheta025=cossqtheta025,cossqtheta975=cossqtheta975,cossqthetadiffmean=cossqthetadiffmean,cossqthetadiff025=cossqthetadiff025,cossqthetadiff975=cossqthetadiff975,diffunormmean=diffunormmean,diffrnormmean=diffrnormmean))
}

#plot log abundances +1 against year
#note we discard data from the first year, which will be strongly influenced by initialization of the Kalman filter
#year is vector of times
#abundances is array of abundances (rows are years, columns are species
#yl is vertical axis limit
#panel.label is text for top left-hand corner
#xoff is offset of label from left edge
#cexax is size for axis tick labels
#cexlab is size for axis labels
#col is colour for lines
plotlograw <- function(year,abundances,yl,panel.label,xoff=0.01,cexax=1.5,cexlab=1.5,col=colors()[320]){
  nspp <- dim(abundances)[2]
  plot(year[-1],log(abundances[-1,1]),type="n",ylim=yl,xlab="Year",ylab=bquote(log(italic(c[i])+1)),cex.lab=cexlab,cex.axis=cexax)
  for(i in 1:nspp){
    lines(year[-1],log(abundances[-1,i]+1),col=col)
  }
  axl <- par("usr")
  text(axl[1]+xoff*(axl[2]-axl[1]),axl[4]-0.07*(axl[4]-axl[3]),panel.label,cex=1.5,pos=4)
}

#plot rbar against time (note first year skipped because will be influenced by initialization of state-space model)
#year is vector of times
#sssuc is output from getsucc
#panel.label is text for top left-hand panel
plotrbar <- function(year,sssuc,panel.label){
  yl <- range(c(sssuc$rstats$rbar025,sssuc$rstats$rbar975),na.rm=TRUE)
  plot(year[-1],c(NA,sssuc$rstats$rbarmean[-1]),type="n",xlab="Year",ylab=bquote('Size'~change~(years^-1)),ylim=yl,cex.lab=1.5,cex.axis=1.5)

  px <- c(year[c(-1,-length(year))]+1)
  px <- c(px,rev(px))
  py <- c(sssuc$rstats$rbar025[-1],rev(sssuc$rstats$rbar975[-1]))
  polygon(px,py,col=colors()[320],border=NA)
  lines(year[-1],c(NA,sssuc$rstats$rbarmean[-1]))
  axl <- par("usr")
  text(axl[1]+0.01*(axl[2]-axl[1]),axl[4]-0.07*(axl[4]-axl[3]),panel.label,cex=1.5,pos=4)
}

#plot sr against time (note first year skipped because will be influenced by initialization of state-space model)
#year is vector of times
#sssuc is output from getsucc
#panel.label is text for top left-hand panel
plotsr <- function(year,sssuc,panel.label){
  yl <- range(c(sssuc$rstats$sr025,sssuc$rstats$sr975),na.rm=TRUE)
  plot(year[-1],c(NA,sssuc$rstats$srmean[-1]),type="n",xlab="Year",ylab=bquote('Shape'~change~(years^-1)),ylim=yl,cex.lab=1.5,cex.axis=1.5)
  px <- c(year[c(-1,-length(year))]+1)
  px <- c(px,rev(px))
  py <- c(sssuc$rstats$sr025[-1],rev(sssuc$rstats$sr975[-1]))
  polygon(px,py,col=colors()[320],border=NA)
  lines(year[-1],c(NA,sssuc$rstats$srmean[-1]))
  axl <- par("usr")
  text(axl[1]+0.01*(axl[2]-axl[1]),axl[4]-0.07*(axl[4]-axl[3]),panel.label,cex=1.5,pos=4)
}

#changes in scaled mean and sd of mean proportional growth rate (lines starting from origin)
#sssuc is output from getsucc
#nspp is number of species
#panel.label is text for top left-hand corner
plotchangeangles <- function(sssuc,nspp,panel.label){
  lr <- length(sssuc$rstats$rbarmean)
  x <- diff(sssuc$rstats$rbarmean*sqrt(nspp))
  y <- diff(sssuc$rstats$srmean*sqrt(nspp-1))
  xl <- c(-max(abs(x)),max(abs(x)))
  yl <- c(-max(abs(y)),max(abs(y)))
  plot(x,y,type="n",asp=1,xlim=xl,ylim=yl,cex.lab=1.5,cex.axis=1.5,xlab=bquote(Delta(italic(n)^{1/2}~bar(italic(r)))),ylab=bquote(Delta(italic(n)-1)^{1/2}~italic(s[r])))
  for(i in 1:(lr-1)){
    lines(x=c(0,x[i]),y=c(0,y[i]))
  }
  axl <- par("usr")
  text(axl[1]+0.01*(axl[2]-axl[1]),axl[4]-0.07*(axl[4]-axl[3]),panel.label,cex=1.5,pos=4)
}

#scaled mean and sd of mean proportional growth rate over time
#semicircles show norm of growth rate vector
#sssuc is output from getsucc
#nspp is number of species
#panel.label is text for top left-hand corner
plotchangecircle <- function(sssuc,nspp,panel.label){
  colors <- brewer.pal(5,"Dark2")[2:3]
  lr <- length(sssuc$rstats$rbarmean)
  yl <- c(0,10)
  plot(sssuc$rstats$rbarmean*sqrt(nspp),sssuc$rstats$srmean*sqrt(nspp-1),type="n",asp=1,xlab=bquote(italic(n)^{1/2}~bar(italic(r))),ylab=bquote((italic(n)-1)^{1/2}~italic(s[r])),cex.lab=1.5,cex.axis=1.5,ylim=yl,yaxs="i")
  theta <- c(seq(from=0,to=pi,length.out=1000))
  for(r in 1:12){
    x <- r*cos(theta)
    y <- r*sin(theta)
    lines(t(x),t(y),lty="dashed",col=colors()[320])
  }

#blue when rbar decreasing, orange when increasing
  for(i in 1:(lr-1)){
    if(sssuc$rstats$rbarmean[i+1]<sssuc$rstats$rbarmean[i]) col=colors[2]
    else if (sssuc$rstats$rbarmean[i+1]>sssuc$rstats$rbarmean[i]) col=colors[1]
    else col="black"
    lines(x=sssuc$rstats$rbarmean[i:(i+1)]*sqrt(nspp),y=sssuc$rstats$srmean[i:(i+1)]*sqrt(nspp-1),col=col,lwd=1)
    if(i==1) points(sssuc$rstats$rbarmean[1]*sqrt(nspp),sssuc$rstats$srmean[1]*sqrt(nspp-1),pch=16,col=col)
  }
  points(sssuc$rstats$rbarmean[lr]*sqrt(nspp),sssuc$rstats$srmean[lr]*sqrt(nspp-1),pch=1,col=col)
  axl <- par("usr")
  text(axl[1]+0.01*(axl[2]-axl[1]),axl[4]-0.07*(axl[4]-axl[3]),panel.label,cex=1.5,pos=4)
}

#check my angle calculations for changes in u and r
#NOTE WE NO LONGER CALCULATE COSTHETADIFFMEAN
checkangles <- function(sssuc,nspp){
  x <- sssuc$rstats$costhetadiffmean
  y <- sqrt(1-x^2)

  
  x3 <- sssuc$rstats$diffunormmean
  y3 <- sqrt(sssuc$rstats$diffrnormmean^2-x3^2)

  xl <- c(0,1,0,1)
  yl <- c(0,0,1,1)
  plot(xl,yl,asp=1,type="n")
  points(x,y,col="red",pch=16,cex=0.5) #bootstrap mean cosine
  lr <- length(sssuc$rstats$rbarmean)
  x2 <- abs(diff(sssuc$rstats$rbarmean*sqrt(nspp)))
  y2 <- abs(diff(sssuc$rstats$srmean*sqrt(nspp-1)))
  for(i in 1:(lr-1)){
    lines(x=c(0,10*x2[i]),y=c(0,10*y2[i])) #my original angles folded into first quadrant
    lines(x=c(0,10*x3[i]),y=c(0,10*y3[i]),col="red") #angles from bootstrap means (shows that difference between original angles and bootstrap mean cosine isn't just because of nonlinearity)
  }

}

#decompose changes in growth rate vector into a norm and an acute angle relative to the line of equal changes
#sssuc is output from getsucc
#nspp is number of species
#panel.label is text for top left-hand corner
#NOTE WE NO LONGER CALCULATE COSTHETADIFFMEAN
plotchangecomponents <- function(sssuc,nspp,panel.label){
  plot(1/sqrt(nspp)*sssuc$rstats$diffrnormmean,sssuc$rstats$costhetadiffmean,ylim=c(0,1),yaxp=c(0,1,2),ylab=bquote(cos~theta),xlab=bquote(italic(n)^{-1/2}~group("||",Delta~bold(r),"||")),cex.lab=1.5,cex.axis=1.5)
  axl <- par("usr")
  text(axl[1]+0.01*(axl[2]-axl[1]),axl[4]-0.07*(axl[4]-axl[3]),panel.label,cex=1.5,pos=4)

}

#``activity'' vs proportion of change that is "size" change
#sssuc is output from getsucc
#nspp is number of species
#panel.label is text for top left-hand corner
plottempvprop <- function(sssuc,nspp,panel.label){
  plot(1/sqrt(nspp)*sssuc$rstats$rnormmean,sssuc$rstats$cossqthetamean,ylim=c(0,1),yaxp=c(0,1,2),ylab=bquote(Proportion~'"size"'~change),xlab=bquote(Scaled~activity~(years^-1)),cex.lab=1.5,cex.axis=1.5)
  axl <- par("usr")
  text(axl[1]+0.01*(axl[2]-axl[1]),axl[4]-0.07*(axl[4]-axl[3]),panel.label,cex=1.5,pos=4)  
}
